Multiscroll.Tool.Scroll = function(parent) {
	this.parent = parent;	// to monitor clientX, clientY

	this.count = 0;
	this.markerNode = {
		image : null,
		container : null
	};
	this.all = {
		parent : this,
		get image() { return this.parent.image.all; },
		get cursor() { return this.parent.cursor.all; },
	};
	this.vertical = {
		parent : this,
		get image() { return this.parent.image.vertical; },
		get cursor() { return this.parent.cursor.vertical; },
	};
	this.horizontal = {
		parent : this,
		get image() { return this.parent.image.horizontal; },
		get cursor() { return this.parent.cursor.horizontal; },
	};
	this.type getter= function() {
		return this.windowHeight < this.documentHeight && this.windowWidth >= this.documentWidth ? this.vertical
			: this.windowHeight >= this.documentHeight && this.windowWidth < this.documentWidth ? this.horizontal
			: this.all;
	};
};

Multiscroll.Tool.Scroll.prototype = new Multiscroll.Tool({
	start : function(evt) {
		this.active = true;
		this.done = true;
		
		if (this.settings.marker)
			this.insertMarker(evt);
		
		setTimeout(this.loop, 5, this);
	},
	stop : function(evt) { 
		this.active = false;
		this.removeMarker(evt);
	},
	loop : function(scroll) { with (scroll.parent.event) {
		if (scroll.active) {
			var x = (clientX - scroll.startX) * scroll.settings.multiplier;
			var y = (clientY - scroll.startY) * scroll.settings.multiplier;
			x = x>0 ? Math.pow(Math.abs(x), scroll.settings.exponent) : x<0 ? -Math.pow(Math.abs(x), scroll.settings.exponent) : 0;
			y = y>0 ? Math.pow(Math.abs(y), scroll.settings.exponent) : y<0 ? -Math.pow(Math.abs(y), scroll.settings.exponent) : 0;
			
			if (Math.abs(x) > 0 && Math.abs(y) > 0) {	// I do not expect to understand this
				if (scroll.count++ % 2)
					y = 0;
				else
					x = 0;
			}
			scroll.scrollX += x;
			scroll.scrollY += y;
			scroll.client.frame.scrollTo(scroll.scrollX, scroll.scrollY);
			if (Math.abs(scroll.scrollX - scroll.client.frame.pageXOffset) > 10 )
				scroll.scrollX = scroll.client.frame.pageXOffset;
			if (Math.abs(scroll.scrollY - scroll.client.frame.pageYOffset) > 10 )
				scroll.scrollY = scroll.client.frame.pageYOffset;
			// retain sub-pixel movements but protect edges from exponential dragging
			setTimeout(scroll.loop, 5, scroll);
		}
	} },
	//	The following submitted by: Erik Arvidsson erik@eae.net 
	//	Slightly modified by: Matthew Ratzloff matt@kanchi.org
	//	http://www.mozdev.org/pipermail/autoscroll/2002-December/000002.html
	insertMarker : function(evt) {
		var docBox = this.client.document.getBoxObjectFor(this.client.document.documentElement);
		var left = evt.screenX - docBox.screenX;
		var top = evt.screenY - docBox.screenY;
		
		this.documentWidth = docBox.width;
		this.documentHeight = docBox.height;
		this.windowWidth = window.innerWidth;
		this.windowHeight = window.innerHeight;
		var imageWidth = 28;
		var imageHeight = 28;
	
		// overlay
		var el = this.client.document.createElementNS("http://www.w3.org/1999/xhtml", "div");
		el.style.position = "fixed";
		el.style.left = "0px";
		el.style.top = "0px";
		el.style.width = this.documentWidth + "px";
		el.style.height = this.documentHeight + "px";
		el.style.cursor = this.type.cursor;
	
		this.client.body.appendChild(el);
		this.markerNode.container = el;
	
		// marker
		el = this.client.document.createElementNS("http://www.w3.org/1999/xhtml", "img");
		el.src = this.type.image;
	
		el.style.position = "fixed";
		el.style.left = left - imageWidth / 2 + "px";
		el.style.top = top - imageHeight / 2 + "px";
		el.style.width = imageWidth + "px";
		el.style.height = imageHeight + "px";
		el.style.cursor = this.type.cursor;
	
		this.client.body.appendChild(el);
	
		this.markerNode.image = el;
	},
	removeMarker : function() {
		if (this.markerNode.image != null)
			this.markerNode.image.parentNode.removeChild(this.markerNode.image);
	
		if (this.markerNode.container != null)
			this.markerNode.container.parentNode.removeChild(this.markerNode.container);
		
		this.markerNode.container = null;
		this.markerNode.image = null;
	}
});